﻿using System;
using System.Globalization;
using FormatNet.Fakes;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace FormatNet
{
	[TestClass]
	public class FormatterTest
	{
		[TestMethod]
		[ExpectedException( typeof( ArgumentNullException ) )]
		public void Format_WithNullFormat_ShouldThrowArgumentNullException()
		{
			var obj = new object();
			var formatter = Formatter.CreateFormatter();
			formatter.Format( null, obj );
		}

		[TestMethod]
		public void Format_WithNullObject_ShouldReturnEmptyText()
		{
			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{0}", null );
			Assert.AreEqual( string.Empty, actual );
		}

		[TestMethod]
		public void Format_WithNullObject_ShouldReturnDefaultIfNullText()
		{
			const string expected = "null";
			var formatter = Formatter.CreateFormatter();
			formatter.DefaultIfNull( expected );
			var actual = formatter.Format( "{0}", null );
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_WithNullObjectAndPropertyInFormatString_ShouldReturnEmptyText()
		{
			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{FirstName}", null );
			Assert.AreEqual( string.Empty, actual );
		}

		[TestMethod]
		public void Format_WithNullObjectAndPropertyChainInFormatString_ShouldReturnEmptyText()
		{
			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{Child.FirstName}", null );
			Assert.AreEqual( string.Empty, actual );
		}

		[TestMethod]
		public void Format_WithNullProperty_ShouldReturnEmptyText()
		{
			var obj = new Person { FirstName = null };
			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{FirstName}", obj );
			Assert.AreEqual( string.Empty, actual );
		}

		[TestMethod]
		public void Format_WithNullPropertyAndPropertyChainInFormatString_ShouldReturnEmptyText()
		{
			var obj = new Person { Child = null };
			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{Child.FirstName}", obj );
			Assert.AreEqual( string.Empty, actual );
		}

		[TestMethod]
		public void Format_WithNullPropertyInChainAndPropertyChainInFormatString_ShouldReturnEmptyText()
		{
			var obj = new Person { Child = new Person { FirstName = null } };
			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{Child.FirstName}", obj );
			Assert.AreEqual( string.Empty, actual );
		}

		[TestMethod]
		public void Format_WithEmptyObjectArray_ShouldReturnTokenlessFormatString()
		{
			const string expected = "test";
			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( expected, new object[0] );
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		[ExpectedException( typeof( FormatException ) )]
		public void Format_WithInvalidFormat_ShouldThrowFormatException()
		{
			var obj = new object();
			var formatter = Formatter.CreateFormatter();
			formatter.Format( "{Name}", obj );
		}

		[TestMethod]
		public void Format_PassingValidFormatString_ShouldReturnFormattedObject()
		{
			var obj = new Person
			{
				FirstName = "Riccardo",
				LastName = "Gregori",
				BirthDay = new DateTime( 1983, 12, 22 )
			};

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{FirstName} {LastName}, nato il {BirthDay:dd/MM/yyyy}", obj );
			const string expected = "Riccardo Gregori, nato il 22/12/1983";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingZero_ShouldReturnObjectToString()
		{
			var obj = new Stub( "Sample text" );

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{0} sample!", obj );
			const string expected = "Sample text sample!";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingNegativeAlignment_ShouldLeftAlignText()
		{
			var obj = new Person
			{
				FirstName = "Riccardo"
			};

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{FirstName,-20}", obj );
			const string expected = "Riccardo            ";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingPositiveAlignment_ShouldRightAlignText()
		{
			var obj = new Person
			{
				FirstName = "Riccardo"
			};

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{FirstName,20}", obj );
			const string expected = "            Riccardo";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingPositionFieldAndAligment_ShouldReturnFormattedString()
		{
			var obj = new Person
			{
				FirstName = "Riccardo"
			};

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{0.FirstName,-20}", obj );
			const string expected = "Riccardo            ";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingChildPropertyWithDoubleDot_ShouldNotMatch1()
		{
			var obj = new Person { FirstName = "Marco", Child = new Person { FirstName = "Riccardo" } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{Child..FirstName}", obj );

			Assert.AreEqual( "{Child..FirstName}", actual );
		}

		[TestMethod]
		public void Format_PassingChildPropertyWithDoubleDot_ShouldNotMatch2()
		{
			var obj = new Person { FirstName = "Marco", Child = new Person { FirstName = "Riccardo" } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{Child.FirstName..}", obj );

			Assert.AreEqual( "{Child.FirstName..}", actual );
		}

		[TestMethod]
		public void Format_PassingChildPropertyWithDoubleDot_ShouldNotMatch3()
		{
			var obj = new Person { FirstName = "Marco", Child = new Person { FirstName = "Riccardo" } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{..Child.FirstName}", obj );

			Assert.AreEqual( "{..Child.FirstName}", actual );
		}

		[TestMethod]
		public void Format_PassingChildProperty_ShouldReturnChildPropertyValue()
		{
			var obj = new Person { FirstName = "Marco", Child = new Person { FirstName = "Riccardo" } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{Child.FirstName}", obj );

			Assert.AreEqual( obj.Child.FirstName, actual );
		}

		[TestMethod]
		public void Format_PassingPositionAndChildProperty_ShouldReturnChildPropertyValue()
		{
			var obj1 = new Object();
			var obj2 = new Person { FirstName = "Marco", Child = new Person { FirstName = "Riccardo" } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{1.Child.FirstName}", obj1, obj2 );

			Assert.AreEqual( obj2.Child.FirstName, actual );
		}

		[TestMethod]
		public void Format_PassingPositionAndChildPropertyAndAlignment_ShouldReturnAlignedChildPropertyValue()
		{
			var obj1 = new Object();
			var obj2 = new Person { FirstName = "Marco", Child = new Person { FirstName = "Riccardo" } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{1.Child.FirstName,-20}", obj1, obj2 );

			const string expected = "Riccardo            ";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingChildPropertyAndFormat_ShouldReturnFormattedChildPropertyValue()
		{
			var obj = new Person { FirstName = "Marco", Child = new Person { BirthDay = new DateTime( 1983, 12, 22 ) } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{Child.BirthDay:dd/MM/yyyy}", obj );

			const string expected = "22/12/1983";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingPositionAndChildPropertyAndFormat_ShouldReturnFormattedChildPropertyValue()
		{
			var obj1 = new Object();
			var obj2 = new Person { FirstName = "Marco", Child = new Person { BirthDay = new DateTime( 1983, 12, 22 ) } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{1.Child.BirthDay:dd/MM/yyyy}", obj1, obj2 );

			const string expected = "22/12/1983";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingChildPropertyAndFormatAndAligment_ShouldReturnFormattedAndAlignedChildPropertyValue()
		{
			var obj2 = new Person { FirstName = "Marco", Child = new Person { BirthDay = new DateTime( 1983, 12, 22 ) } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{Child.BirthDay,-20:dd/MM/yyyy}", obj2 );

			const string expected = "22/12/1983          ";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_PassingPositionAndChildPropertyAndFormatAndAligment_ShouldReturnFormattedAndAlignedChildPropertyValue()
		{
			var obj1 = new Object();
			var obj2 = new Person { FirstName = "Marco", Child = new Person { BirthDay = new DateTime( 1983, 12, 22 ) } };

			var formatter = Formatter.CreateFormatter();
			var actual = formatter.Format( "{1.Child.BirthDay,-20:dd/MM/yyyy}", obj1, obj2 );

			const string expected = "22/12/1983          ";
			Assert.AreEqual( expected, actual );
		}

		[TestMethod]
		public void Format_UsingItalianCultureInfo_ShouldReturnLocalizedString()
		{
			var formatter = Formatter.CreateFormatter().UseFormatProvider( new CultureInfo( "it-IT" ) );
			var actual = formatter.Format( "{0:dddd}", new DateTime( 2010, 12, 4 ) );

			var expected = "sabato";
			Assert.AreEqual( expected, actual );
		}
	}
}
